
import uk.gov.nhse.vascard.*;
import java.util.*;

class SigningExampleDefinitions
{

	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a raw byte array 
	/// representation of the PKCS#1v1.5 Digital Signature. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputRawByteArray(HSVASCardSign objsign, String readerName, String pinNumber, int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = 0;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		
		try
		{

			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0xA0;
				throw new Exception("One or more calling arguments is invalid");
			}
			
			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed");
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a byte array of the raw PKCS#1 v1.5 Digital Signature.
			SignatureResult digitalSignature = new SignatureResult();
			res = objsign.SignHashGeneratingPKCS1v1point5ByteArray(readerName,
						pinNumber,
						cCTX,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.signature));

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}

		return res;
	}

	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a byte array containing a Base64 
	/// representation of the PKCS#1v1.5 Digital Signature. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputBase64ByteArray(HSVASCardSign objsign, String readerName, String pinNumber, int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = SCardSignOptions.BASE64;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example. On success this example returns a byte array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");		

		try
		{

			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0xA0;
				throw new Exception("One or more calling arguments is invalid");
			}
			
			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed");
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a byte array of the raw PKCS#1 v1.5 Digital Signature.
			SignatureResult digitalSignature = new SignatureResult();
			res = objsign.SignHashGeneratingPKCS1v1point5ByteArray(readerName,
						pinNumber,
						cCTX,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.signature));

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		System.out.println("\n<Single Digital Signing Operation example. On success this example returns a byte array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");	

		return res;
	}
	
	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a raw byte array 
	/// representation of the PKCS#1v1.5 Digital Signature. It uses the Key Container name to perform the signing
	/// operation. However, this would require the signing key container name on the card to be known to the application.
	/// Therefore, this example looks that up first before calling the signing method. A developer would not want to do
	/// this in the real world as other methods are more appropriate, this would only be used in cases where the 
	/// key container is always the same for the application. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputRawByteArrayByKeyContainerName(HSVASCardSign objsign,String readerName, String pinNumber, int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = 0;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example using the key container name. On success this example returns a raw byte array representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
			
			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0x0A;
				throw new Exception("One or more calling argments is invalid");
			}

			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
			
			//Get the Smart Card Key Container Name associated with the X.509 Certificate.
			//This will be used in SignHashGeneratingPKCS1v1point5StringEx
			StringBuffer keyContainerName = new StringBuffer();
			bRes = objsign.GetKeyContainerName(cCTX, keyContainerName);
			if(!bRes)
			{
				throw new Exception("GetKeyContainerName failed");
			}
			
			System.out.println("[Key Container Name] : " + keyContainerName.toString());
			
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed");
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a base64 formatted string represenation of the 
			//PKCS#1 v1.5 Digital Signature.
			SignatureResult digitalSignature = new SignatureResult();
			res = objsign.SignHashGeneratingPKCS1v1point5ByteArrayEx(readerName,
						keyContainerName.toString(),
						pinNumber,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5StringEx failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.signature));

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		
		}	

		System.out.println("\n</Single Digital Signing Operation example using the key container name. On success this example returns a raw byte array representation of the PKCS#1 v1.5 Digital Signature>\n");

		return res;
	}	


	
	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a string with the 
	/// hexidecimal representation of the PKCS#1v1.5 Digital Signature. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputHexString(	HSVASCardSign objsign,String readerName, String pinNumber,int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = 0;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example. On success this example returns a Hex String representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
			
			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0xA0;
				throw new Exception("One or more calling arguments is invalid");
			}

			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed");
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a base64 formatted string represenation of the 
			//PKCS#1 v1.5 Digital Signature.
			StringBuffer digitalSignature = new StringBuffer();
			res = objsign.SignHashGeneratingPKCS1v1point5String(readerName,
						pinNumber,
						cCTX,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5String failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.toString().getBytes()));
			System.out.println("[PKCS#1v15 Signature (string)]\n" + digitalSignature.toString());

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		
		}	
		System.out.println("</Single Digital Signing Operation example. On success this example returns a Hex String representation of the PKCS#1 v1.5 Digital Signature>\n");

		return res;
	}


	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a single line Base64 string 
	/// representation of the PKCS#1v1.5 Digital Signature. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputBase64String(HSVASCardSign objsign,String readerName, String pinNumber, int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = SCardSignOptions.BASE64;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example. On success this example returns a single line Base64 String representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
			
			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0xA0;
				throw new Exception("One or more calling argments is invalid");
			}

			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a base64 formatted string represenation of the 
			//PKCS#1 v1.5 Digital Signature.
			StringBuffer digitalSignature = new StringBuffer();
			res = objsign.SignHashGeneratingPKCS1v1point5String(readerName,
						pinNumber,
						cCTX,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5String failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.toString().getBytes()));
			System.out.println("[PKCS#1v15 Signature (string)]\n" + digitalSignature.toString());

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		
		}	

		System.out.println("\n</Single Digital Signing Operation example. On success this example returns a single line Base64 String representation of the PKCS#1 v1.5 Digital Signature>\n");	

		return res;
	}


	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a multiline Base64 string 
	/// representation of the PKCS#1v1.5 Digital Signature. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputBase64LineBreakString(HSVASCardSign objsign,String readerName, String pinNumber, int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = SCardSignOptions.BASE64_LINEBREAKS;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example. On success this example returns a multiline Base64 String representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
			
			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0xA0;
				throw new Exception("One or more calling argments is invalid");
			}

			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
			

			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a base64 formatted string represenation of the 
			//PKCS#1 v1.5 Digital Signature.
			StringBuffer digitalSignature = new StringBuffer();
			res = objsign.SignHashGeneratingPKCS1v1point5String(readerName,
						pinNumber,
						cCTX,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5String failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.toString().getBytes()));
			System.out.println("[PKCS#1v15 Signature (string)]\n" + digitalSignature.toString());

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		
		}	

		System.out.println("\n</Single Digital Signing Operation example. On success this example returns a multiline Base64 String representation of the PKCS#1 v1.5 Digital Signature>\n");	
	
		return res;
	
	}



	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a multiline Base64 string 
	/// representation of the PKCS#1v1.5 Digital Signature. It uses the Key Container name to perform the signing
	/// operation. However, this would require the signing key container name on the card to be known to the application.
	/// Therefore, this example looks that up first before calling the signing method. A developer would not want to do
	/// this in the real world as other methods are more appropriate, this would only be used in cases where the 
	/// key container is always the same for the application. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputBase64LineBreakStringByKeyContainerName(HSVASCardSign objsign,String readerName, String pinNumber,int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = SCardSignOptions.BASE64_LINEBREAKS;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example using the key container name. On success this example returns a multiline Base64 String representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
			
			if(objsign == null || readerName == null || pinNumber == null)
			{
				throw new Exception("One or more calling argments is invalid");
			}

			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
			
			//Get the Smart Card Key Container Name associated with the X.509 Certificate.
			//This will be used in SignHashGeneratingPKCS1v1point5StringEx
			StringBuffer keyContainerName = new StringBuffer();
			bRes = objsign.GetKeyContainerName(cCTX, keyContainerName);
			if(!bRes)
			{
				throw new Exception("GetKeyContainerName failed");
			}
			
			System.out.println("[Key Container Name] : " + keyContainerName.toString());
			
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a base64 formatted string represenation of the 
			//PKCS#1 v1.5 Digital Signature.
			StringBuffer digitalSignature = new StringBuffer();
			res = objsign.SignHashGeneratingPKCS1v1point5StringEx(readerName,
						keyContainerName.toString(),
						pinNumber,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5StringEx failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.toString().getBytes()));
			System.out.println("[PKCS#1v15 Signature (string)]\n" + digitalSignature.toString());

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		
		}	

		System.out.println("\n</Single Digital Signing Operation example using the key container name. On success this example returns a multiline Base64 String representation of the PKCS#1 v1.5 Digital Signature>\n");

		return res;
	}


	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a raw byte array 
	/// representation of the PKCS#1v1.5 Digital Signature. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputRawByteArray_AuthCert(HSVASCardSign objsign, String readerName, String pinNumber,int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = 0;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		
		try
		{

			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0xA0;
				throw new Exception("One or more calling arguments is invalid");
			}
			
			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain theAuthentication Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheAuthenticationCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Authentication Certificate]\n" + X509Base64.toString());
						
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed");
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a byte array of the raw PKCS#1 v1.5 Digital Signature.
			SignatureResult digitalSignature = new SignatureResult();
			res = objsign.SignHashGeneratingPKCS1v1point5ByteArray(readerName,
						pinNumber,
						cCTX,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.signature));

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheAuthenticationCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		return res;
	}
	
	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a byte array containing a Base64 
	/// representation of the PKCS#1v1.5 Digital Signature. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputBase64ByteArray_AuthCert(HSVASCardSign objsign, String readerName, String pinNumber,int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = SCardSignOptions.BASE64;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example. On success this example returns a byte array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");		

		try
		{

			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0xA0;
				throw new Exception("One or more calling argments is invalid");
			}
			
			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain theAuthentication Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheAuthenticationCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheAuthenticationCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Authentication Certificate]\n" + X509Base64.toString());
						
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a byte array of the raw PKCS#1 v1.5 Digital Signature.
			SignatureResult digitalSignature = new SignatureResult();
			res = objsign.SignHashGeneratingPKCS1v1point5ByteArray(readerName,
						pinNumber,
						cCTX,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.signature));

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheAuthenticationCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		System.out.println("\n<Single Digital Signing Operation example. On success this example returns a byte array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");	
		return res;

	}
	
	///<Summary>
	///
	/// This method performs a single Digitial Signing operation which, on success returns a raw byte array 
	/// representation of the PKCS#1v1.5 Digital Signature. It uses the Key Container name to perform the signing
	/// operation. However, this would require the authentication key container name on the card to be known to the application.
	/// Therefore, this example looks that up first before calling the signing method. A developer would not want to do
	/// this in the real world as other methods are more appropriate, this would only be used in cases where the 
	/// key container is always the same for the application. This method expects the signing environment
	/// has already been initialized. 
	///
	///</Summary>
	public static int SmartCardSigningOutputRawByteArrayByKeyContainerName_AuthCert(HSVASCardSign objsign,String readerName, String pinNumber,int hashAlgorithm)
	{

		int res = 0;
		SCardCertStore cStore =  null;
		SCardCertContext cCTX = null;
		int options = 0;
		byte[] inputData = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin".getBytes();
		
		System.out.println("<Single Digital Signing Operation example using the key container name. On success this example returns a raw byte array representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
			
			if(objsign == null || readerName == null || pinNumber == null)
			{
				res = 0x0A;
				throw new Exception("One or more calling argments is invalid");
			}

			//Open the certificate store for the Smart Card.
			cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
			//Obtain theAuthentication Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheAuthenticationCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheAuthenticationCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
			//Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Authentication Certificate]\n" + X509Base64.toString());
			
			//Get the Smart Card Key Container Name associated with the X.509 Certificate.
			//This will be used in SignHashGeneratingPKCS1v1point5StringEx
			StringBuffer keyContainerName = new StringBuffer();
			bRes = objsign.GetKeyContainerName(cCTX, keyContainerName);
			if(!bRes)
			{
				throw new Exception("GetKeyContainerName failed");
			}
			
			System.out.println("[Key Container Name] : " + keyContainerName.toString());
			
			//Generate a hash digest to digitally sign. This can be done via the this API or another
			//hasing library as long as you map its algorithm Identifiers to this API.
			
			HashResult result = new HashResult();
			res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
			if(res != 0 )
			{
				throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
			System.out.println("[Hash Size] : " + result.hashValue.length);
			System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));

			//Digitally Sign the hash returning a base64 formatted string represenation of the 
			//PKCS#1 v1.5 Digital Signature.
			SignatureResult digitalSignature = new SignatureResult();
			res = objsign.SignHashGeneratingPKCS1v1point5ByteArrayEx(readerName,
						keyContainerName.toString(),
						pinNumber,
						options,
						hashAlgorithm,
						result.hashValue,
						digitalSignature);
			if(res != 0 )
			{
				throw new Exception("SignHashGeneratingPKCS1v1point5StringEx failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("\n[PKCS#1v15 Signature (Hex Dump)]\n" + ExampleUtilities.HexDumpWithASCIItoString(digitalSignature.signature));

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		//Release the allocated resourses. This must be done to avoid memory leaks from the JNI world.
		if(objsign != null)
		{
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheAuthenticationCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		
		}	

		System.out.println("\n</Single Digital Signing Operation example using the key container name. On success this example returns a raw byte array representation of the PKCS#1 v1.5 Digital Signature>\n");
		
		return res;
	}
	

	///<Summary>
	///
	/// This method performs a Bulk Digitial Signing operation which, when a successful signing operation is complete returns a raw byte array 
	/// representation of the PKCS#1v1.5 Digital Signature. The example performs multiple digital signing operations in a loop.
	/// This example method expects the signing environment has already been initialized. 
	/// 
	///</Summary>
	public static int BulkDigitalSigningOutputRawByteArray(HSVASCardSign objsign, String readerName, String pinNumber,int hashAlgorithm, int signatureCount)
	{

		int res = 0;
		SCardCertStore cStore = null;
		SCardCertContext cCTX = null;
		SCardSigningSession signingSession = null;
		int options = 0;
        String inputPartial = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin"; 

		System.out.println("<Bulk Digital Signing Operation example. On success this example returns a Raw Byte Array representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
	
			if(objsign == null || readerName == null || pinNumber == null || signatureCount == 0)
			{
				res = 0xA0;
				throw new Exception("One or more calling arguments is invalid");
			}

            //Open the certificate store for the Smart Card.
            cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
            //Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
            //Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Initialize a signing session which allows for multiple Digital Signing requests
            //to be performed. This is required for better processing of repeat signing operations.

			signingSession  = new SCardSigningSession();
			res = objsign.InitSigningSession(readerName,pinNumber,cCTX,options,signingSession);
			if(res != 0 )
			{
				throw new Exception("InitSigningSession failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Perform " + signatureCount +" PKCS#1v15 Signature Operations with " + HashAlgorithm.getHashName(hashAlgorithm) + " Hashes]\n");
			//perform 10 digital signing requests 
			for(int x = 0 ; x < signatureCount ; x++ )
			{
			
                //Generate a hash digest to digitally sign. This can be done via the this API or another
			    //hasing library as long as you map its algorithm Identifiers to this API.
				HashResult result = new HashResult();
				String tmp = "Hash [" + (x+1) + "]. " + inputPartial;
				byte[] inputData = tmp.getBytes();
				
				System.out.println("[Data to hash] : " + tmp);
				
				res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
				if(res != 0 )
				{
					throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
				System.out.println("[Hash Size] : " + result.hashValue.length);
				System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));


				SignatureResult signatureResult = new SignatureResult();
				res = objsign.SigningSessionSignHashGeneratingPKCS1v1point5ByteArray(signingSession,
							cCTX,
							options,
							hashAlgorithm,
							result.hashValue,
							signatureResult);
				if(res != 0 )
				{
					throw new Exception("SigningSessionSignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[PKCS#1v15 Signature]\n");
				System.out.println(ExampleUtilities.HexDumpWithASCIItoString(signatureResult.signature));
			}
			System.out.println("[Completed executing PKCS#1v15 Signature Operations with " + HashAlgorithm.getHashName(hashAlgorithm) +  " Hashes]\n");

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		if(objsign != null)
		{
			if(signingSession.GetHandle() != 0)
			{
				//Uninitialize the Digital Signing Session
				objsign.UninitSigningSession(signingSession);
			}
			
			if(cCTX.GetHandle() != 0)
			{
				//Release the digital signing certificate
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				//Close the Smart Card Store
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		}
		
		System.out.println("\n</Bulk Digital Signing Operation example completed. Return Code: " + String.format("0x%08X", res) + ">\n");

		return res;

	}

	///<Summary>
	///
	/// This method performs a Bulk Digitial Signing operation which, when a successful signing operation is complete returns a byte array 
	/// containing a Base64 representation of the PKCS#1v1.5 Digital Signature. The example performs multiple digital signing operations 
	/// in a loop. This example method expects the signing environment has already been initialized. 
	/// 
	///</Summary>
	public static int BulkDigitalSigningOutputBase64ByteArray(HSVASCardSign objsign, String readerName, String pinNumber,int hashAlgorithm, int signatureCount)
	{

		int res = 0;
		SCardCertStore cStore = null;
		SCardCertContext cCTX = null;
		SCardSigningSession signingSession = null;
		int options = SCardSignOptions.BASE64;
        String inputPartial = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin"; 

		System.out.println("<Bulk Digital Signing Operation example. On success this example returns a Byte Array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
	
			if(objsign == null || readerName == null || pinNumber == null || signatureCount == 0)
			{
				res = 0xA0;
				throw new Exception("One or more calling arguments is invalid");
			}

            //Open the certificate store for the Smart Card.
            cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
            //Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
            //Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Initialize a signing session which allows for multiple Digital Signing requests
            //to be performed. This is required for better processing of repeat signing operations.

			signingSession  = new SCardSigningSession();
			res = objsign.InitSigningSession(readerName,pinNumber,cCTX,options,signingSession);
			if(res != 0 )
			{
				throw new Exception("InitSigningSession failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Perform " + signatureCount +" PKCS#1v15 Signature Operations with " + HashAlgorithm.getHashName(hashAlgorithm) + " Hashes]\n");
			//perform 10 digital signing requests 
			for(int x = 0 ; x < signatureCount ; x++ )
			{
			
                //Generate a hash digest to digitally sign. This can be done via the this API or another
			    //hasing library as long as you map its algorithm Identifiers to this API.
				HashResult result = new HashResult();
				String tmp = "Hash [" + (x+1) + "]. " + inputPartial;
				byte[] inputData = tmp.getBytes();
				
				System.out.println("[Data to hash] : " + tmp);
				
				res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
				if(res != 0 )
				{
					throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
				System.out.println("[Hash Size] : " + result.hashValue.length);
				System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));


				SignatureResult signatureResult = new SignatureResult();
				res = objsign.SigningSessionSignHashGeneratingPKCS1v1point5ByteArray(signingSession,
							cCTX,
							options,
							hashAlgorithm,
							result.hashValue,
							signatureResult);
				if(res != 0 )
				{
					throw new Exception("SigningSessionSignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[PKCS#1v15 Signature]\n");
				System.out.println(ExampleUtilities.HexDumpWithASCIItoString(signatureResult.signature));
			}
			System.out.println("[Completed executing PKCS#1v15 Signature Operations with " + HashAlgorithm.getHashName(hashAlgorithm) +  " Hashes]\n");
			


		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		if(objsign != null)
		{
			if(signingSession.GetHandle() != 0)
			{
				//Uninitialize the Digital Signing Session
				objsign.UninitSigningSession(signingSession);
			}
			
			if(cCTX.GetHandle() != 0)
			{
				//Release the digital signing certificate
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				//Close the Smart Card Store
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		System.out.println("\n</Bulk Digital Signing Operation example. On success this example returns a Byte Array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");
	
		return res;
	}

	///<Summary>
	///
	/// This method performs a Bulk Digitial Signing operation which, when a successful signing operation is complete returns a raw byte array 
	/// representation of the PKCS#1v1.5 Digital Signature. The InitSigningSessionEx method is used to initialize the session in this example,
	/// this method takes the key container name that holds the digital signing cetificate required for digital signing. To provide this name
	/// the example gets the certificate and the associated key container name but in reality you'd not want to use it this way, it only does
	/// this as the key container name for a test card being used will be unknown. You'd be expected to obtain the key container name through
	/// another mechanism such as winscard. This example just shows how this InitSigningSessionEx method works.
	/// The example performs multiple digital signing operations in a loop.This example method expects the signing environment has already 
	/// been initialized. 
	/// 
	///</Summary>
	public static int BulkDigitalSigningExOutputRawByteArray(HSVASCardSign objsign, String readerName, String pinNumber, int hashAlgorithm, int signatureCount)
	{

		int res = 0;
		SCardCertStore cStore = null;
		SCardCertContext cCTX = null;
		SCardSigningSession signingSession = null;
		int options = 0;
        String inputPartial = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin"; 

		System.out.println("<Bulk Digital Signing Operation example using InitSigningSessionEx. On success this example returns a Byte Array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
	
			if(objsign == null || readerName == null || pinNumber == null || signatureCount == 0)
			{
				res = 0xA0;
				throw new Exception("One or more calling arguments is invalid");
			}

            //Open the certificate store for the Smart Card.
            cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
            //Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
            //Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Get the Smart Card Key Container Name associated with the X.509 Certificate.
			//This is required as the example will not know the key container name for the
			//card being used. There are more appropriate methods to use than this and youd
			//only want to use the InitSigningSessionEx version if you knew the container
			//name.
			StringBuffer keyContainerName = new StringBuffer();
			bRes = objsign.GetKeyContainerName(cCTX, keyContainerName);
			if(!bRes)
			{
				throw new Exception("GetKeyContainerName failed");
			}
			
			System.out.println("[Key Container Name] : " + keyContainerName.toString());


			//Initialize a signing session which allows for multiple Digital Signing requests
            //to be performed. This is required for better processing of repeat signing operations.

			signingSession  = new SCardSigningSession();
			res = objsign.InitSigningSessionEx(readerName,keyContainerName.toString(),pinNumber,options,signingSession);
			if(res != 0 )
			{
				throw new Exception("InitSigningSession failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Perform " + signatureCount +" PKCS#1v15 Signature Operations with " + HashAlgorithm.getHashName(hashAlgorithm) + " Hashes]\n");
			//perform 10 digital signing requests 
			for(int x = 0 ; x < signatureCount ; x++ )
			{
			
                //Generate a hash digest to digitally sign. This can be done via the this API or another
			    //hasing library as long as you map its algorithm Identifiers to this API.
				HashResult result = new HashResult();
				String tmp = "Hash [" + (x+1) + "]. " + inputPartial;
				byte[] inputData = tmp.getBytes();
				
				System.out.println("[Data to hash] : " + tmp);
				
				res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
				if(res != 0 )
				{
					throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
				System.out.println("[Hash Size] : " + result.hashValue.length);
				System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));


				SignatureResult signatureResult = new SignatureResult();
				res = objsign.SigningSessionSignHashGeneratingPKCS1v1point5ByteArray(signingSession,
							cCTX,
							options,
							hashAlgorithm,
							result.hashValue,
							signatureResult);
				if(res != 0 )
				{
					throw new Exception("SigningSessionSignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[PKCS#1v15 Signature]\n");
				System.out.println(ExampleUtilities.HexDumpWithASCIItoString(signatureResult.signature));
			}
			System.out.println("[Completed executing PKCS#1v15 Signature Operations with " + HashAlgorithm.getHashName(hashAlgorithm) +  " Hashes]\n");
		
		

		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;
			System.out.println("Exception occurred: " + ex);
		}
		
		if(objsign != null)
		{
			if(signingSession.GetHandle() != 0)
			{
				//Uninitialize the Digital Signing Session
				objsign.UninitSigningSession(signingSession);
			}
			
			if(cCTX.GetHandle() != 0)
			{
				//Release the digital signing certificate
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				//Close the Smart Card Store
				objsign.CloseSmartCardCertificateStore(cStore);
			}
		}

		System.out.println("\n<Bulk Digital Signing Operation example using InitSigningSessionEx. On success this example returns a Byte Array containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		return res;
	}
	
	public static int BulkSigningSessionOutputHexString(HSVASCardSign objsign,String readerName, String pinNumber,int hashAlgorithm,int signatureCount)
	{

		int res = 0;
		SCardCertStore cStore = null;
		SCardCertContext cCTX = null;
		SCardSigningSession signingSession = null;
		int options = 0;

        String inputPartial = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin"; 

		System.out.println("<Bulk Digital Signing Operation example. On success this example returns a String containing a Hex representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{	
			
			if(objsign == null || readerName == null || pinNumber == null || signatureCount == 0 )
			{
				res = 0xA0;	
				throw new Exception("One or more calling argments is invalid");
			}
			

            //Open the certificate store for the Smart Card.
            cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
            //Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
            //Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
			
			
			//Initialize a signing session which allows for multiple Digital Signing requests
            //to be performed. This is required for better processing of repeat signing operations.

			signingSession  = new SCardSigningSession();
			res = objsign.InitSigningSession(readerName,pinNumber,cCTX,options,signingSession);
			if(res != 0 )
			{
				throw new Exception("InitSigningSession failed. Error Code: " + String.format("0x%08X", res));
			}
			
			//perform 10 digital signing requests 
			for(int x = 0 ; x < signatureCount ; x++ )
			{
			
                //Generate a hash digest to digitally sign. This can be done via the this API or another
			    //hasing library as long as you map its algorithm Identifiers to this API.
				HashResult result = new HashResult();
				String tmp = "Hash [" + (x+1) + "]. " + inputPartial;
				byte[] inputData = tmp.getBytes();
				
				System.out.println("[Data to hash] : " + tmp);
				
				res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
				if(res != 0 )
				{
					throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
				System.out.println("[Hash Size] : " + result.hashValue.length);

				System.out.println("[Hash Value] : " + ExampleUtilities.BytesToHex(result.hashValue));


				StringBuffer digitalSignature = new StringBuffer();
				res = objsign.SigningSessionSignHashGeneratingPKCS1v1point5String(signingSession,
							cCTX,
							options,
							hashAlgorithm,
							result.hashValue,
							digitalSignature);
				if(res != 0 )
				{
					throw new Exception("SigningSessionSignHashGeneratingPKCS1v1point5String failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[PKCS#1v15 Signature]\n" + digitalSignature.toString());
			}
		
		
		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;	
			System.out.println("Exception occurred: " + ex);
		}
		
		if(objsign != null)
		{
			if(signingSession.GetHandle() != 0)
			{
				objsign.UninitSigningSession(signingSession);
			}
			
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		System.out.println("</Bulk Digital Signing Operation example. On success this example returns a String containing a Hex representation of the PKCS#1 v1.5 Digital Signature>\n");
		
		return res;
	}

	public static int BulkSigningSessionOutputBase64String(HSVASCardSign objsign,String readerName, String pinNumber,int hashAlgorithm,int signatureCount)
	{

		int res = 0;
		SCardCertStore cStore = null;
		SCardCertContext cCTX = null;
		SCardSigningSession signingSession = null;
		int options = SCardSignOptions.BASE64;

        String inputPartial = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin"; 

		System.out.println("<Bulk Digital Signing Operation example. On success this example returns a String containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{	
			
			if(objsign == null || readerName == null || pinNumber == null || signatureCount == 0 )
			{
				res = 0xA0;		
				throw new Exception("One or more calling argments is invalid");
			}
			

            //Open the certificate store for the Smart Card.
            cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
            //Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
            //Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
			
			
			//Initialize a signing session which allows for multiple Digital Signing requests
            //to be performed. This is required for better processing of repeat signing operations.

			signingSession  = new SCardSigningSession();
			res = objsign.InitSigningSession(readerName,pinNumber,cCTX,options,signingSession);
			if(res != 0 )
			{
				throw new Exception("InitSigningSession failed. Error Code: " + String.format("0x%08X", res));
			}
			
			//perform 10 digital signing requests 
			for(int x = 0 ; x < signatureCount ; x++ )
			{
			
                //Generate a hash digest to digitally sign. This can be done via the this API or another
			    //hasing library as long as you map its algorithm Identifiers to this API.
				HashResult result = new HashResult();
				String tmp = "Hash [" + (x+1) + "]. " + inputPartial;
				byte[] inputData = tmp.getBytes();
				
				System.out.println("[Data to hash] : " + tmp);
				
				res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
				if(res != 0 )
				{
					throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
				System.out.println("[Hash Size] : " + result.hashValue.length);

				System.out.println("[Hash Value] : " + ExampleUtilities.BytesToHex(result.hashValue));


				StringBuffer digitalSignature = new StringBuffer();
				res = objsign.SigningSessionSignHashGeneratingPKCS1v1point5String(signingSession,
							cCTX,
							options,
							hashAlgorithm,
							result.hashValue,
							digitalSignature);
				if(res != 0 )
				{
					throw new Exception("SigningSessionSignHashGeneratingPKCS1v1point5String failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[PKCS#1v15 Signature]\n" + digitalSignature.toString());
			}
		
		
		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;	
			System.out.println("Exception occurred: " + ex);
		}
		
		if(objsign != null)
		{
			if(signingSession.GetHandle() != 0)
			{
				objsign.UninitSigningSession(signingSession);
			}
			
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		System.out.println("\n</Bulk Digital Signing Operation example. On success this example returns a String containing a Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		return res;
	}	

	public static int BulkSigningSessionOutputBase64LineBreakString(HSVASCardSign objsign,String readerName, String pinNumber,int hashAlgorithm,int signatureCount)
	{

		int res = 0;
		SCardCertStore cStore = null;
		SCardCertContext cCTX = null;
		SCardSigningSession signingSession = null;
		int options = SCardSignOptions. BASE64_LINEBREAKS;

        String inputPartial = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin"; 

		System.out.println("<Bulk Digital Signing Operation example. On success this example returns a String containing a line broken Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{	
			
			if(objsign == null || readerName == null || pinNumber == null || signatureCount == 0 )
			{
				res = 0xA0;
				throw new Exception("One or more calling argments is invalid");
			}
			

            //Open the certificate store for the Smart Card.
            cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
            //Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
            //Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
			
			
			//Initialize a signing session which allows for multiple Digital Signing requests
            //to be performed. This is required for better processing of repeat signing operations.

			signingSession  = new SCardSigningSession();
			res = objsign.InitSigningSession(readerName,pinNumber,cCTX,options,signingSession);
			if(res != 0 )
			{
				throw new Exception("InitSigningSession failed. Error Code: " + String.format("0x%08X", res));
			}
			
			//perform 10 digital signing requests 
			for(int x = 0 ; x < signatureCount ; x++ )
			{
			
                //Generate a hash digest to digitally sign. This can be done via the this API or another
			    //hasing library as long as you map its algorithm Identifiers to this API.
				HashResult result = new HashResult();
				String tmp = "Hash [" + (x+1) + "]. " + inputPartial;
				byte[] inputData = tmp.getBytes();
				
				System.out.println("[Data to hash] : " + tmp);
				
				res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
				if(res != 0 )
				{
					throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
				System.out.println("[Hash Size] : " + result.hashValue.length);

				System.out.println("[Hash Value] : " + ExampleUtilities.BytesToHex(result.hashValue));


				StringBuffer digitalSignature = new StringBuffer();
				res = objsign.SigningSessionSignHashGeneratingPKCS1v1point5String(signingSession,
							cCTX,
							options,
							hashAlgorithm,
							result.hashValue,
							digitalSignature);
				if(res != 0 )
				{
					throw new Exception("SigningSessionSignHashGeneratingPKCS1v1point5String failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[PKCS#1v15 Signature]\n" + digitalSignature.toString());
			}
		
		
		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;				
			System.out.println("Exception occurred: " + ex);
		}
		
		if(objsign != null)
		{
			if(signingSession.GetHandle() != 0)
			{
				objsign.UninitSigningSession(signingSession);
			}
			
			if(cCTX.GetHandle() != 0)
			{
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		System.out.println("\n</Bulk Digital Signing Operation example. On success this example returns a String containing a line broken Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		return res;

	}	

	//<Summary>
	///
	/// This method performs a Bulk Digitial Signing operation which, when a successful signing operation is complete returns a raw byte array 
	/// representation of the PKCS#1v1.5 Digital Signature. The InitSigningSessionEx method is used to initialize the session in this example,
	/// this method takes the key container name that holds the digital signing cetificate required for digital signing. To provide this name
	/// the example gets the certificate and the associated key container name but in reality you'd not want to use it this way, it only does
	/// this as the key container name for a test card being used will be unknown. You'd be expected to obtain the key container name through
	/// another mechanism such as winscard. This example just shows how this InitSigningSessionEx method works.
	/// The example performs multiple digital signing operations in a loop.This example method expects the signing environment has already 
	/// been initialized. 
	/// 
	///</Summary>	
	public static int BulkDigitalSigningExOutputBase64LineBreakString(HSVASCardSign objsign, String readerName, String pinNumber,int hashAlgorithm, int signatureCount)
	{

		int res = 0;
		SCardCertStore cStore = null;
		SCardCertContext cCTX = null;
		SCardSigningSession signingSession = null;
		int options = SCardSignOptions.BASE64_LINEBREAKS;
        String inputPartial = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras tortor nunc, posuere tempus nulla quis, imperdiet fringilla proin"; 

		System.out.println("<Bulk Digital Signing Operation example using InitSigningSessionEx. On success this example returns a String containing a line broken Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		try
		{
	
			if(objsign == null || readerName == null || pinNumber == null || signatureCount == 0)
			{
				res = 0xA0;	
				throw new Exception("One or more calling arguments is invalid");
			}

            //Open the certificate store for the Smart Card.
            cStore = new SCardCertStore();
			res = objsign.OpenSmartCardCertificateStore(readerName,cStore,SCardSignOptions.NONE);
			if(res != 0 )
			{
				throw new Exception("OpenSmartCardCertificateStore failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Cert Store Opened]");
		
            //Obtain the Digital Signing Certificate from the Smart Card.
			cCTX = new SCardCertContext();
			res = objsign.ObtainTheSigningCertificate(cStore,cCTX,options);
			if(res != 0 )
			{
				throw new Exception("ObtainTheSigningCertificate failed. Error Code: " + String.format("0x%08X", res));
			}

			System.out.println("*** Obtained the Signing Certificate ***");
		
            //Get the Base64 representation of the X.509 Certificate. 
			StringBuffer X509Base64 = new StringBuffer();
			boolean bRes = objsign.CertificateContextToBase64ASN1(cCTX,X509Base64, Base64CertFormat.CRLF);
			if(!bRes)
			{
				throw new Exception("CertificateContextToBase64ASN1 failed.");
			}
			
			System.out.println("[Signing Certificate]\n" + X509Base64.toString());
						
			//Get the Smart Card Key Container Name associated with the X.509 Certificate.
			//This is required as the example will not know the key container name for the
			//card being used. There are more appropriate methods to use than this and youd
			//only want to use the InitSigningSessionEx version if you knew the container
			//name.
			StringBuffer keyContainerName = new StringBuffer();
			bRes = objsign.GetKeyContainerName(cCTX, keyContainerName);
			if(!bRes)
			{
				throw new Exception("GetKeyContainerName failed. Error Code: " + String.format("0x%08X", res));
			}
			
			System.out.println("[Key Container Name] : " + keyContainerName.toString());


			//Initialize a signing session which allows for multiple Digital Signing requests
            //to be performed. This is required for better processing of repeat signing operations.

			signingSession  = new SCardSigningSession();
			res = objsign.InitSigningSessionEx(readerName,keyContainerName.toString(),pinNumber,options,signingSession);
			if(res != 0 )
			{
				throw new Exception("InitSigningSession failed. Error Code: " + String.format("0x%08X", res));
			}
			
			//perform 10 digital signing requests 
			for(int x = 0 ; x < signatureCount ; x++ )
			{
			
                //Generate a hash digest to digitally sign. This can be done via the this API or another
			    //hasing library as long as you map its algorithm Identifiers to this API.
				HashResult result = new HashResult();
				String tmp = "Hash [" + (x+1) + "]. " + inputPartial;
				byte[] inputData = tmp.getBytes();
				
				System.out.println("[Data to hash] : " + tmp);
				
				res = objsign.GenerateHashDigest(hashAlgorithm,inputData, result);
				if(res != 0 )
				{
					throw new Exception("GenerateHashDigest failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[Hash Algorithm] : " + HashAlgorithm.getHashName(result.algorithm));
				System.out.println("[Hash Size] : " + result.hashValue.length);
				System.out.println("[Hash Value]\n" + ExampleUtilities.HexDumpWithASCIItoString(result.hashValue));


				StringBuffer signatureResult = new StringBuffer();
				res = objsign.SigningSessionSignHashGeneratingPKCS1v1point5String(signingSession,
							cCTX,
							options,
							hashAlgorithm,
							result.hashValue,
							signatureResult);
				if(res != 0 )
				{
					throw new Exception("SigningSessionSignHashGeneratingPKCS1v1point5ByteArray failed. Error Code: " + String.format("0x%08X", res));
				}

				System.out.println("[PKCS#1v15 Signature]\n");
				System.out.println(signatureResult.toString());
			}
		
		
		}
		catch(Exception ex)
		{
			if(res == 0)
				res = 0x54F;				
			System.out.println("Exception occurred: " + ex);
		}
		
		if(objsign != null)
		{
			if(signingSession.GetHandle() != 0)
			{
				//Uninitialize the Digital Signing Session
				objsign.UninitSigningSession(signingSession);
			}
			
			if(cCTX.GetHandle() != 0)
			{
				//Release the digital signing certificate
				objsign.ReleaseTheSigningCertificate(cCTX);
			}
			
			if(cStore.GetHandle() != 0)
			{
				//Close the Smart Card Store
				objsign.CloseSmartCardCertificateStore(cStore);
			}		
		}	

		System.out.println("\n</Bulk Digital Signing Operation example using InitSigningSessionEx. On success this example returns a String containing a line broken Base64 representation of the PKCS#1 v1.5 Digital Signature>\n");

		return res;
	}

	

}